@@ -187,8 +187,12 @@ static void init_dicts() {
 
 Image::sf_t Rule::Cache::parseSampleFormat(char const*s, slen_t slen) {
   /* Tue Jul  2 13:48:12 CEST 2002 */
-  static char rule_dummy=(init_dicts(),0); /* call once per process */
-  (void)rule_dummy;
+  #if 0 /* BUGFIX for g++-3.4 (needs symbols __cxa_guard_acquire, __cxa_guard_release) */
+    static char rule_dummy=(init_dicts(),0); /* call once per process */
+    (void)rule_dummy;
+  #else
+    if (y_FileFormat==NULLP) init_dicts();
+  #endif
   while (slen!=0 && s[0]=='/') { s++; slen--; }
   if (slen<=0 || slen>=SampleFormat_MAXLEN) return Image::SF_max;
   char buf[SampleFormat_MAXLEN];
@@ -293,7 +297,8 @@ char const* Rule::CacheHints::dumpScale(sc_t Scale) {
  */
 static MiniPS::VALUE better_predictor(MiniPS::VALUE v) {
   return v==MiniPS::Qundef ? MiniPS::Qinteger(1) /* PR_None */
-       : v==MiniPS::Qinteger(1) || v==MiniPS::Qinteger(2) || v==MiniPS::Qinteger(45) ||
+       : v==MiniPS::Qinteger(1) || v==MiniPS::Qinteger(2) ||
+         v==MiniPS::Qinteger(45) || v==MiniPS::Qinteger(55) ||
          ((v&1)!=0 && v>=MiniPS::Qinteger(10) && v<=MiniPS::Qinteger(15)) ? v
        : MiniPS::Qundef;
 }
@@ -330,8 +335,12 @@ static MiniPS::VALUE better_colorTransform(MiniPS::VALUE v) {
 }
 
 void Rule::OutputRule::fromDict(MiniPS::VALUE dict_) {
-  static char rule_dummy=(init_dicts(),0); /* call once per process */
-  (void)rule_dummy;
+  #if 0 /* BUGFIX for g++-3.4 (needs symbols __cxa_guard_acquire, __cxa_guard_release) */
+    static char rule_dummy=(init_dicts(),0); /* call once per process */
+    (void)rule_dummy;
+  #else
+    if (y_FileFormat==NULLP) init_dicts();
+  #endif
   MiniPS::VALUE dummy;  (void)dummy;
   MiniPS::VALUE FileFormat, SampleFormat, WarningOK, TransferEncoding,
     Compression, Predictor, Transparent;
@@ -587,8 +596,9 @@ Rule::OutputRule* Rule::buildProfile(MiniPS::VALUE Profile, bool quiet) {
   #if !USE_BUILTIN_LZW
     bool lzw_warning=true;
   #endif
-  if (quiet) Error::pushPolicy((Error::level_t)0, Error::WARNING, Error::WARNING_DEFER, (GenBuffer::Writable*)NULLP);
-  
+  if (quiet) Error::pushPolicy((Error::level_t)0,
+   /*printed_:*/Error::getTopPrinted()+0>Error::WARNING+0 ? Error::getTopPrinted() : Error::WARNING,
+   Error::WARNING_DEFER, (GenBuffer::Writable*)NULLP); /* Dat: WARNING_DEFER untested */
   for (c=0, pary->getFirst(val); val!=NULLP; pary->getNext(val), c++) {
     /* val: each OutputRule of the Profile */
     or_->fromDict(*val);
@@ -626,7 +636,9 @@ Rule::OutputRule* Rule::buildProfile(MiniPS::VALUE Profile, bool quiet) {
 void Rule::applyProfile(GenBuffer::Writable& out, OutputRule*rule_list, Image::SampledInfo *sf) {
   OutputRule *or_;
   // unsigned tryc=0; /* Wed Jul  3 19:30:33 CEST 2002 */
-  Error::pushPolicy((Error::level_t)0, Error::NOTICE, Error::NOTICE_DEFER, (GenBuffer::Writable*)NULLP);
+  Error::pushPolicy((Error::level_t)0,
+    /*printed_:*/Error::getTopPrinted()+0>Error::NOTICE+0 ? Error::getTopPrinted() : Error::NOTICE,
+    Error::NOTICE_DEFER, (GenBuffer::Writable*)NULLP);
   Image::Sampled::rgb_t Transparent=0x1000000UL;
   if (rule_list->dict!=NULLP) {
     Transparent=rule_list->cache.Transparent;
@@ -868,7 +880,7 @@ void Rule::writeTTE(
       out << "0 " << i;
       while (j--!=1) out << " 0 " << i;
       break;
-     case 'D': /* 0..1 /Decode values for indexed, 0..max-1 for others */
+     case 'D': /* 0..max-1 /Decode values for indexed, 0..1 for others */
       i=(or_->cache.isIndexed())?(1<<img->getBpc())-1:1;
       goto emit_Decode;
      case 'S': /* image data stream */
